#define GSL_SIZEOF_PTH_MUTEX_T		(40)
#define GSL_SIZEOF_PTH_COND_T		(48)
#define GSL_HAVE_MUTEXATTR_SETTYPE	(1 && \
					GSL_SIZEOF_PTH_MUTEX_T && \
					GSL_SIZEOF_PTH_COND_T)
#define GSL_SIZEOF_STD_INTMAX_T		(8)
#define GSL_HAVE_LIBMAD			(1)
#define GSL_HAVE_OGGVORBIS		(1)
#define GSL_HAVE_OGGVORBIS_RC3		(0)
#define GSL_USE_GSL_GLIB		(1)
#define GSL_USE_ARTS_THREADS		(1)
